;;;
;;; Allen's Interval Algebra (AIA) with 13 base relations
;;;

(def-calculus "Allen's Interval Algebra (AIA IA)"
  :arity :binary
  :parametric? nil
  :consistency :scenario-consistency
  :qualifier #'(lambda (p1 p2)
		 (let ((s1 (first p1))
		       (e1 (second p1))
		       (s2 (first p2))
		       (e2 (second p2)))
		   (cond ((< s1 s2) (cond ((< e1 s2) 'B)
					  ((= e1 s2) 'M)
					  (t (cond ((< e1 e2) 'O)
						   ((= e1 e2) 'FI)
						   (t 'DI)))))
			 ((= s1 s2) (cond ((< e1 e2) 'S)
					  ((= e1 e2) 'EQ)
					  (t 'SI)))
			 (t (cond ((< e2 s1) 'BI)
				  ((= e2 s1) 'MI)
				  (t (cond ((< e2 e1) 'OI)
					   ((= e2 e1) 'F)
					   (t 'D))))))))

  :basis-entity :interval
  :identity-relation EQ
  :converse-operation ((EQ EQ)
		       (B  BI)
		       (BI B )
		       (D  DI)
		       (DI D )
		       (O  OI)
		       (OI O )
		       (M  MI)
		       (MI M )
		       (S  SI)
                       (SI S)
		       (F  FI)
		       (FI F ))
  
  :base-relations (EQ B BI D DI O OI M MI S SI F FI)
  :composition-operation ((EQ EQ	EQ)
			  (EQ B	        B)
			  (EQ BI	BI)
			  (EQ D		D)
			  (EQ DI	DI)
			  (EQ O		O)
			  (EQ OI	OI)
			  (EQ M		M)
			  (EQ MI	MI)
			  (EQ S		S)
			  (EQ SI	SI)
			  (EQ F		F)
			  (EQ FI	FI)
			  
			  (B  EQ	B)
			  (B  B		B)
			  (B  BI	(EQ B BI D DI O OI M MI S SI F FI))
			  (B  D		(B O M D S))
			  (B  DI	B)
			  (B  O		B)
			  (B  OI	(B O M D S))
			  (B  M		B)
			  (B  MI	(B O M D S))
			  (B  S		B)
			  (B  SI	B)
			  (B  F		(B O M D S))
			  (B  FI	B)
			  
			  (BI EQ	BI)
			  (BI B		(EQ B BI D DI O OI M MI S SI F FI))
			  (BI BI	BI)
			  (BI D		(BI OI MI D F))
			  (BI DI	BI)
			  (BI O		(BI OI MI D F))
			  (BI OI	BI)
			  (BI M		(BI OI MI D F))
			  (BI MI	BI)
			  (BI S		(BI OI MI D F))
			  (BI SI	BI)
			  (BI F		BI)
			  (BI FI	BI)
			  
			  (D  EQ	D)
			  (D  B		B)
			  (D  BI	BI)
			  (D  D		D)
			  (D  DI	(EQ B BI D DI O OI M MI S SI F FI))
			  (D  O		(B O M D S))
			  (D  OI	(BI OI MI D F))
			  (D  M		B)
			  (D  MI	BI)
			  (D  S		D)
			  (D  SI	(BI OI MI D F))
			  (D  F		D)
			  (D  FI	(B O M D S))
			  
			  (DI EQ	DI)
			  (DI B		(B O M DI FI))
			  (DI BI	(BI OI DI MI SI))
			  (DI D		(O OI D DI EQ S SI F FI))
			  (DI DI	DI)
			  (DI O		(O DI FI))
			  (DI OI	(OI DI SI))
			  (DI M		(O DI FI))
			  (DI MI	(OI DI SI))
			  (DI S		(O DI FI))
			  (DI SI	DI)
			  (DI F		(OI DI SI))
			  (DI FI	DI)
			  
			  (O  EQ	O)
			  (O  B		B)
			  (O  BI	(BI OI DI MI SI))
			  (O  D		(O D S))
			  (O  DI	(B O M DI FI))
			  (O  O		(B O M))
			  (O  OI	(O OI D DI EQ S SI F FI))
			  (O  M		B)
			  (O  MI	(OI DI SI))
			  (O  S		O)
			  (O  SI	(O DI FI))
			  (O  F		(O D S))
			  (O  FI	(B O M))
			  
			  (OI EQ	OI)
			  (OI B		(B O M DI FI))
			  (OI BI	BI)
			  (OI D		(OI D F))
			  (OI DI	(BI OI DI MI SI))
			  (OI O		(O OI D DI EQ S SI F FI))
			  (OI OI	(BI OI MI))
			  (OI M		(O DI FI))
			  (OI MI	BI)
			  (OI S		(OI D F))
			  (OI SI	(BI OI MI))
			  (OI F		OI)
			  (OI FI	(OI DI SI))
			  
			  (M  EQ	M)
			  (M  B		B)
			  (M  BI	(BI OI DI MI SI))
			  (M  D		(O D S))
			  (M  DI	B)
			  (M  O		B)
			  (M  OI	(O D S))
			  (M  M		B)
			  (M  MI	(F FI EQ))
			  (M  S		M)
			  (M  SI	M)
			  (M  F		(O D S))
			  (M  FI	B)
			  
			  (MI EQ	MI)
			  (MI B		(B O M DI FI))
			  (MI BI	BI)
			  (MI D		(OI D F))
			  (MI DI	BI)
			  (MI O		(OI D F))
			  (MI OI	BI)
			  (MI M		(S SI EQ))
			  (MI MI	BI)
			  (MI S		(OI D F))
			  (MI SI	BI)
			  (MI F		MI)
			  (MI FI	MI)
	
			  (S  EQ	S)
			  (S  B		B)
			  (S  BI	BI)
			  (S  D		D)
			  (S  DI	(B O M DI FI))
			  (S  O		(B O M))
			  (S  OI	(OI D F))
			  (S  M		B)
			  (S  MI	MI)
			  (S  S		S)
			  (S  SI	(S SI EQ))
			  (S  F		D)
			  (S  FI	(B O M))
			  
			  (SI EQ	SI)
			  (SI B		(B O M DI FI))
			  (SI BI	BI)
			  (SI D		(OI D F))
			  (SI DI	DI)
			  (SI O		(O DI FI))
			  (SI OI	OI)
			  (SI M		(O DI FI))
			  (SI MI	MI)
			  (SI S		(S SI EQ))
			  (SI SI	SI)
			  (SI F		OI)
			  (SI FI	DI)
			  
			  
			  (F  EQ	F)
			  (F  B		B)
			  (F  BI	BI)
			  (F  D		D)
			  (F  DI	(BI OI DI MI SI))
			  (F  O		(O D S))
			  (F  OI	(BI OI MI))
			  (F  M		M)
			  (F  MI	BI)
			  (F  S		D)
			  (F  SI	(BI OI MI))
			  (F  F		F)
			  (F  FI	(F FI EQ))
			  	
			  (FI EQ	FI)
			  (FI B		B)
			  (FI BI	(BI OI DI MI SI))
			  (FI D		(O D S))
			  (FI DI	DI)
			  (FI O		O)
			  (FI OI	(OI SI DI))
			  (FI M		M)
			  (FI MI	(OI SI DI))
			  (FI S		O)
			  (FI SI	DI)
			  (FI F		(F FI EQ))
			  (FI FI	FI)
			  )
  :algebraic-specification ((*  ((1 ((a1 1))) < (1 ((a2 1))))  ; for all relations: A,B are proper non-empty intervals
				((1 ((b1 1))) < (1 ((b2 1)))))
			    (EQ ((1 ((a1 1))) = (1 ((b1 1))))
				((1 ((a2 1))) = (1 ((b2 1))))) ; equality
			    (b ((1 ((a2 1))) < (1 ((b1 1)))))  ; A before B
			    (bi ((1 ((b2 1))) < (1 ((a1 1))))) ; B before A
			    (d ((1 ((a1 1))) > (1 ((b1 1))))   ; A during B
			       ((1 ((a2 1))) < (1 ((b2 1)))))
			    (di ((1 ((a1 1))) < (1 ((b1 1))))  ; B during A
				((1 ((a2 1))) > (1 ((b2 1)))))
			    (o ((1 ((a1 1))) < (1 ((b1 1))))   ; A overlaps B
			       ((1 ((a2 1))) > (1 ((b1 1))))
			       ((1 ((a2 1))) < (1 ((b2 1)))))
			    (oi ((1 ((b1 1))) < (1 ((a1 1))))  ; B overlaps A
				((1 ((b2 1))) > (1 ((a1 1))))
				((1 ((b2 1))) < (1 ((a2 1)))))
			    (m ((1 ((a2 1))) = (1 ((b1 1)))))  ; A meets B
			    (mi ((1 ((b2 1))) = (1 ((a1 1))))) ; B meets A			    
			    (s ((1 ((a1 1))) = (1 ((b1 1))))   ; A starts B
			       ((1 ((a2 1))) < (1 ((b2 1)))))
			    (si ((1 ((b1 1))) = (1 ((a1 1))))  ; B starts A
				((1 ((b2 1))) < (1 ((a2 1)))))
			    (f ((1 ((a1 1))) > (1 ((b1 1))))   ; A finishes B
			       ((1 ((a2 1))) = (1 ((b2 1)))))
			    (fi ((1 ((b1 1))) > (1 ((a1 1))))  ; B finishes A
				((1 ((b2 1))) = (1 ((a2 1))))))
)


